/*
** Module   : Chunk.java
** Abstract : A structure to hold screen chunks.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- --------------------------------Description-----------------------------------
** 001 MAG 20140227 Created initial version.
*/

package com.goldencode.p2j.ui.client.chui.driver.web;

/**
 * This structures holds screen chunks. A chunk is a list of contiguous screen cells 
 * within a line having the same color and attributes that should be send down to JS
 * code and redraw.
 */
public class Chunk
{
   /** Header size */
   public static final int HEADER_SIZE = 2;
   
   /** Start row  */
   private final int row;
   
   /** Start column */
   private final int column;
   
   /** Chunk size */
   private int size;
   
   /** Chunk payload */
   private byte[] chunk;

   /**
    * Constructor.
    * 
    * @param   row
    *          Start row;
    * @param   column
    *          Start column;
    * @param   size
    *          Chunk size.
    */
   public Chunk(int row, int column, int size)
   {
      this.row = row;
      this.column = column;
      this.size = size;
   }

   /**
    * Constructor.
    * 
    * @param   chunk
    *          Chunk payload.
    */
   public Chunk(byte[] chunk)
   {
      this.chunk = chunk;
      this.row = chunk[0];
      this.column = chunk[1];
      this.size = chunk.length;
   }
   
   /**
    * Get chunk payload.
    * 
    * @return  Chunk payload
    */
   public byte[] getChunk()
   {
      return chunk;
   }

   /**
    * Get start row.
    * 
    * @return  row number;
    */
   public int getRow()
   {
      return row;
   }

   /**
    * Get start column.
    * 
    * @return  column number.
    */
   public int getColumn()
   {
      return column;
   }

   /**
    * Get chunk size.
    * 
    * @return  chunk size.
    */
   public int getSize()
   {
      return size;
   }

   /**
    * Get text size.
    * 
    * @return  Text size.
    */
   public int getTextSize()
   {
      return size - Chunk.HEADER_SIZE;
   }
   
   /**
    * Return the unique hash code for this object.
    *
    * @return  Unique hash code for this object.
    */   
   @Override
   public int hashCode()
   {
      final int prime = 31;
      int result = 1;
      result = prime * result + column;
      result = prime * result + row;
      result = prime * result + size;
      return result;
   }

   /**
    * This method returns true if this object is equal to the specified
    * object.  Equality is defined as both objects being instances of this
    * class and the class name being the same.
    *
    * @param   obj
    *          Another Chunk object to check for equality.
    *
    * @return  True if objects are equal, else false.
    */   
   @Override
   public boolean equals(Object obj)
   {
      if (this == obj)
         return true;
      if (obj == null)
         return false;
      if (getClass() != obj.getClass())
         return false;
      Chunk other = (Chunk) obj;
      if (column != other.column)
         return false;
      if (row != other.row)
         return false;
      if (size != other.size)
         return false;
      return true;
   }
}
