/*
** Module   : tty.sockets.js
** Abstract : WebSockets communication management.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
** 002 MAG 20140204 Get attributes as a binary message.
**                  On quit command go back to login page. 
** 003 MAG 20140211 Implements COLOR.
** 004 MAG 20140214 Implements referrer redirection.
** 005 MAG 20140219 Performances improvements.
*/

tty.socket = (function()
{
   /* exports object */
   var my = {}; 

   /* referrer address */
   var referrer;

   /* web socket */
   var ws;
   
   /* connected */
   var connected = false;

   /**
    * Send data.
    * 
    * @param {array} data
    */
   my.send = function(data)
   {
      if (connected) 
      {
         ws.send(data);
      }
      else
      {
         tty.sound.beep();
      }
   };

   /**
    * Initialize module.
    * 
    * @param {object} cfg  configuration.
    */
   my.init = function(cfg)
   {      
      referrer = cfg.referrer;
      
      if ('WebSocket' in window)
      {
         ws = new WebSocket(cfg.socket.url);

         // on web socket open
         ws.onopen = function()
         {
            connected = true;
            ws.binaryType = 'arraybuffer';
         };

         // on web socket message
         ws.onmessage = function(evt)
         {
            var data = evt.data;
            
            if (data instanceof ArrayBuffer)
            {
               // binary messages
               var message = new Uint8Array(data);
               
               switch(message[0])
               {
                  case 0x80:
                     // clear screen 
                     tty.screen.clear();                        
                     break;
                  case 0x81:
                     // draw screen
                     tty.screen.drawRectangles(message);
                     break;
                  case 0x82:
                     // set cursor position
                     tty.screen.setCursorPosition(message[1], message[2]);                        
                     break;
                  case 0x83:
                     // show cursor
                     tty.screen.setCursorStatus(message[1]);                        
                     break;
                  case 0x84:
                     // message beep
                     tty.sound.beep();                        
                     break;
                  case 0x85:
                     // quit
                     window.location.replace(referrer);                        
                     break;
               };               
            }
            else
            {
               // text messages
               var pay = JSON.parse(data);
               
               switch (pay.c)
               {
                  case 0:
                     // color palette
                     tty.screen.palette = pay.p;                        
                     break;
               };
            }
         };

         // on web socket close
         ws.onclose = function()
         { 
            connected = false;
            tty.screen.error('Connection closed by remote host.'); 
            window.setInterval(function()
            {
               window.location.replace(referrer);                        
            }, 5000);
         };
         
         // refresh
         window.onpagehide = function() 
         {
            if (connected)
            {
               ws.close();
            }
         };                  
      } 
      else 
      {
         tty.screen.error('WebSocket NOT supported by your Browser.');
      }      
   };
   
   /* exports */
   return my;       
}());
